/******************************************************************************
*******************************************************************************

	Space Impakto DS
	Relminator
	http://rel.betterwebber.com


	HomingMissile class


*******************************************************************************
******************************************************************************/

#include <nds.h>
#include <vector>

#include "gl2d.h"
#include "Vector2df32.h"
#include "UTIL.h"
#include "GlobalManager.h"

#ifndef HOMINGMISSILE_H
#define HOMINGMISSILE_H

#define HOMING_SPEED ( 2 << 12 )
#define MAX_SEGMENTS 32

class HomingMissile 
{

public:
	struct Link
	{
		s32		X;
		s32		Y;
		Vector2df32 Normal;
	};
	
	
	HomingMissile()
	{
		Active = false;
		Frame = 0;
		NumFrames = 3;
		BaseFrame = 0;
		Width = 20;
		Height = 20;
		Target.x = 256 << 12;
		Target.y = 96 << 12;
		Speed = HOMING_SPEED;
		Radius = 8;
		
		for( int i = 0; i < MAX_SEGMENTS; i++ )
		{
			Segments[i].x = x;
			Segments[i].y = y;
		}
	
		Links.resize( MAX_SEGMENTS/2 );
		
	}
	
	~HomingMissile()
	{
	}
	
	void DrawLinks( int Color, glImage* const Sprites  );
	
	void Init( s32 sx, s32 sy, int frame, int irotation );
	
	void Update();
	
	inline bool IsActive() const
	{
		return Active;
	}
	
	inline void Kill()
	{
		Active = false;
		x = 300 << 12;
	}
	
	inline s32 GetX() const
	{
		return x;
	}
	
	inline s32 GetY() const
	{
		return y;
	}
	
	inline s32 GetWidth() const
	{
		return Width;
	}
	
	inline s32 GetHeight() const
	{
		return Height;
	}
	
	
	inline int GetDrawFrame() const
	{
		return BaseFrame + Frame;
	}
	
	
	inline s32 GetAngle() const
	{
		return Angle;
	}
	
	inline int GetEnergy() const
	{
		return Energy;
	}
	
	inline bool GetIsDone() const
	{
		return IsDone;
	}
	
	inline void SetTarget( s32 _x, s32 _y )
	{
		Target.x = _x;
		Target.y = _y;
	}
	
	inline void SetEnergy( int damage )
	{
		Energy = damage;
	}
	
	inline void SetIsDone( bool value )
	{
		IsDone = value;
	}
	
private:

	bool 	Active;
	int 	Counter;
	
	s32		x;
	s32		y;
	s32		Dx;
	s32		Dy;
	s32		Speed;
	s32		Angle;		
	s32		Delta;
	
	bool	NewTarget;
	
	int 	Energy;
	
	int 	Frame;
	int 	NumFrames;
	int 	BaseFrame;
	
	int 	Width;
	int		Height;
	int 	Radius;
	
	int		InitialRotation;
	bool	IsDone;
	
	Vector2df32 Target;
	
	Vector2df32 Segments[MAX_SEGMENTS];
	std::vector<Link> Links;
	
};

#endif // HOMINGMISSILE_H
