/******************************************************************************
*******************************************************************************

	Bubble Fight EX
	relminator
	http://rel.betterwebber.com


	Rotator class


*******************************************************************************
******************************************************************************/

#include "Rotator.h"

#define MIN_X ( -20 << 12 )
#define MIN_Y ( -18 << 12 )

#define MAX_ROTATORS 16

//------------------------------------------------------------------------------
//	Rotator Enemy
//------------------------------------------------------------------------------
Rotator::Rotator()
{
	Frame = 0;
	Active = false;	
	Hp = 1; 
	DrawMode = EN_NORMAL;
	
}

Rotator::~Rotator()
{
}


void Rotator::Update()
{
	
	Path.Update();
	
	x = Path.GetX();
	y = Path.GetY();
	
	if( x < MIN_X )
	{
		Active = false;
		x = 300 << 12;
		y = 96 << 12;
	}

	Counter++;
	// animate
	if( ( Counter & 3 ) == 0 )
	{
		Frame = (Frame + 1) & 3;
	}

	if( Active ) Targets::Instance()->AddCoord( x, y );
	
}


void Rotator::LoadPath( int _sx, int _sy, int _ex, int _ey, s32 _time, bool _flip )
{
	Path.Init( 16 );
	
	Spline::Coord p;
	
	// start
	p.x = _sx;
	p.y = _sy;
	
	// we should use 2 points
	Path.AddPoint( p );
	Path.AddPoint( p );
	
	p.x = 270;
	p.y = 80;
	Path.AddPoint( p );
	
	p.x = 128;
	p.y = 80;
	if( _flip ) p.y = 196 - p.y;
	Path.AddPoint( p );
	
	p.x = 220;
	p.y = 170;
	if( _flip ) p.y = 196 - p.y;
	Path.AddPoint( p );
	
	p.x = 160;
	p.y = 30;
	if( _flip ) p.y = 196 - p.y;
	Path.AddPoint( p );
	
	p.x = 60;
	p.y = 190;
	if( _flip ) p.y = 196 - p.y;
	Path.AddPoint( p );
	
	p.x = 100;
	p.y = 100;
	if( _flip ) p.y = 196 - p.y;
	Path.AddPoint( p );
	
	p.x = 50;
	p.y = 10;
	if( _flip ) p.y = 196 - p.y;
	Path.AddPoint( p );
	
	
	// close last 2 coords
	p.x = _ex;
	p.y = _ey;
	
	// we should use 2 points
	Path.AddPoint( p );
	Path.AddPoint( p );
	
	Path.SetSpeed(32);
	Path.SetT(_time);
	
}
	
//------------------------------------------------------------------------------
//	Rotator Container
//------------------------------------------------------------------------------
//--------------------------------------
//	Ctor and Dtor
//--------------------------------------
RotatorContainer::RotatorContainer()
{
}
	
RotatorContainer::~RotatorContainer()
{
	Rotators.clear();				// clear the bullets
}


void RotatorContainer::Init( int Palette, glImage* const Sprites )
{
	ColorTable = Palette;		// init needed data for drawing
	Images = Sprites;
	Rotators.resize( MAX_ROTATORS/2 );			// 1/2 of max (speed of iteration)
	Rotators.reserve( MAX_ROTATORS );		// reserve max for speed.  No need to do this for enemies
}

//--------------------------------------
//	Virtual Functions
//--------------------------------------
void RotatorContainer::UpdateEntities()
{
	
	std::vector<Rotator>::iterator iter;
	for( iter = Rotators.begin(); iter != Rotators.end(); ++iter )
	{
		if( iter->Active )
			iter->Update();
	}
	
}

void RotatorContainer::DrawEntities()
{
	
	glColor( RGB15(31,31,31) );
	numEntities = 0;
	std::vector<Rotator>::iterator iter;
	for( iter = Rotators.begin(); iter != Rotators.end(); ++iter )
	{
		if( iter->Active )
		{
			numEntities++;
			if( (iter->DrawMode == EN_WHITE) )
			{
				iter->DrawMode = EN_NORMAL;
				glColor( RGB15(31,0,0) );;
				glSprite( (iter->x >> 12) - 8, (iter->y >> 12) - 8, 
					      GL_FLIP_NONE, &Images[iter->Frame+30] 
					    );
				glColor( RGB15(31,31,31) );
	
			}
			else
			{
				glSprite( (iter->x >> 12) - 8, (iter->y >> 12) - 8, 
						  GL_FLIP_NONE, &Images[iter->Frame+30] 
					    );
			}
			
		}
	}
	
}

bool RotatorContainer::Collide( StarFighter &Star )
{
	std::vector<Rotator>::iterator iter;
	for( iter = Rotators.begin(); iter != Rotators.end(); ++iter )
	{
		if( iter->Active )
		{
			int rad2 = 8 * 8;
			int cx = (iter->x >> 12) - (Star.x >> 12);
			int cy = (iter->y >> 12) - (Star.y >> 12);
			if( ( (cx * cx) + (cy * cy) ) < (rad2 * (1+Star.IsBursting())) )
			{
				if( Star.IsBursting() )
				{
					Vector2df32 p;
					p.x = Star.x;
					p.y = Star.y;
					Particle::Instance()->Spawn( p, 2 );
					Star.IncreaseGauge(15);
					iter->Hp--;
					if( (iter->Hp & 3) == 0 ) Sound::Instance()->PlaySFX(E_SFX_HIT);
					if( iter->Hp < 0 )
					{
						iter->Active = false;
						Star.Score += 33;
						Sound::Instance()->PlaySFX(E_SFX_SMALLEXP);
						// Add some nice explosions
						Explosion::Instance()->Spawn( iter->x >> 12,
													  iter->y >> 12,
													   0,
													   0,
													   Explode::SMALL_2
													 );
							
						iter->x = 600 <<12;
						iter->y = 96 << 12;
						
					}
					if( iter->DrawMode == EN_NORMAL )
					{
						if( (iter->Counter & 1) ) iter->DrawMode = EN_WHITE;
					}
				}
				else
				{
					// kill star
					if( !Star.IsInvincible() ) Star.Kill();
				}
				return true;
			}
		}
		
		// missiles if after star enemy is still active
		if( iter->Active )
		{
			int rad2 = 8 * 8;
			
			if( iter->x < ( 256 << 12 ) )
			{
			
				// Missiles
				for( int i = 0; i < MAX_MISSILES; i++)
				{
					HomingMissile Missile= Star.GetMissile(i);
					
					if( Missile.IsActive() )
					{
						int cx = (iter->x >> 12) - (Missile.GetX() >> 12);
						int cy = (iter->y >> 12) - (Missile.GetY() >> 12);
						if( ( (cx * cx) + (cy * cy) ) < (rad2) )
						{
							Vector2df32 p;
							p.x = Missile.GetX();
							p.y = Missile.GetY();
							Particle::Instance()->Spawn( p, 1 );
							Star.DestroyMissile(i);
							iter->Hp--;
							if( iter->Hp < 0 )
							{
								iter->Active = false;
								Star.Score += 33;
								Sound::Instance()->PlaySFX(E_SFX_SMALLEXP);
								// Add some nice explosions
								Explosion::Instance()->Spawn( iter->x >> 12,
															  iter->y >> 12,
															   0,
															   0,
															   Explode::SMALL_4
															 );
									
								iter->x = 600 <<12;
								iter->y = 96 << 12;
								break;
							}
						}
				
					}
				}
			
			}   // iter x < 
	
		}   // iter active
		
	}
	return false;
}



void RotatorContainer::KillAllEntities()
{
	std::vector<Rotator>::iterator iter;
	for( iter = Rotators.begin(); iter != Rotators.end(); ++iter )
	{
		if( iter->Active )
		{
			iter->Active = false;
			iter->x = 600 << 12;
			iter->y = 96 << 12;
		}
	}
	
}

//--------------------------------------
//	Non-Virtual Functions
//--------------------------------------
void RotatorContainer::Spawn( s32 _time, bool _flip, int _frame )
{
	
	bool SlotFull = true;      		// value to check if all the slots are occupied
	
	std::vector<Rotator>::iterator iter;
	for( iter = Rotators.begin(); iter != Rotators.end(); ++iter )
	{
		if( !iter->Active )		// insert a bullet on a free slot
		{

			iter->Active = true;
			iter->Frame = _frame & 3;
			iter->Counter = 0;
			iter->Hp = 1;
			iter->LoadPath( 400, 96, -300, 96, _time, _flip );
			SlotFull = false;			// there's a free slot so we dont execute the code below
			break;
		
		}
	}
	
	// All the slots are full
	// Only push back a bullet if the slot is full and we have less than numbullets
	if( SlotFull && (Rotators.size() < MAX_ROTATORS) )
	{
		Rotator popcorn;
		popcorn.Active = true;
		popcorn.Frame = _frame & 3;
		popcorn.Counter = 0;
		popcorn.Hp = 1;
		popcorn.LoadPath( 400, 96, -300, 96, _time, _flip );
		Rotators.push_back(popcorn);
	}
	
}



