/******************************************************************************
*******************************************************************************

	Bubble Fight EX
	relminator
	http://rel.betterwebber.com


	StarFighter class


*******************************************************************************
******************************************************************************/

#include <nds.h>
#include <gl2d.h>
#include "Vector2df32.h"
#include "cearn_atan.h"
#include "UTIL.h"
#include "Sound.h"
#include "Palettes.h"
#include "Explosion.h"
#include "HomingMissile.h"
#include "Targets.h"
#include "Particles.h"


#ifndef StarFighter__H
#define StarFighter__H

#define  MAX_MISSILES 8
#define  LASER_GAUGE_FULL (60 * 30)

class StarFighter
{
public:
		enum E_STATE
		{
			E_INVINCIBLE 	= (1 << 0),				// after death or stage start and end
			E_LOCKED 		= (1 << 1)				// stage start/end
		};

		enum E_MODE
		{
			E_ENTER     	= 0,				// stage start
			E_EXIT  		= 1 				// stage end
		};
		
		
		StarFighter();
		~StarFighter();
		
		bool Update( int KeyH, int KeyD );
		void UpdateMissiles();
		void Draw( int Palette, glImage* const Sprites ) const;
		void DrawMissiles( int Palette, glImage* const Sprites, glImage* const Sprites2 );
		void DrawLaserGauge( glImage* const Sprites, glImage* const Sprites2 ) const;
		void NewLife();
		void Kill();
		void SetTargets();
		void KillArmament();
		bool SpawnMissile();
		
		inline bool IsBursting() const
		{
			return ( BurstCounter > 0 );
		}
		
		inline bool	IsInvincible() const
		{
			return State & E_INVINCIBLE;
		}
		
		inline bool	IsLocked() const
		{
			return State & E_LOCKED;
		}
		
		inline HomingMissile GetMissile( int index ) const
		{
			return Missiles[index & (MAX_MISSILES-1)];
		}
		
		inline s32 GetMissileX( int index ) const
		{
			return Missiles[index & (MAX_MISSILES-1)].GetX();
		}
		
		inline s32 GetMissileY( int index ) const
		{
			return Missiles[index & (MAX_MISSILES-1)].GetY();
		}
		
		inline void KillMissile( int index )
		{
			Missiles[index & (MAX_MISSILES-1)].Kill();
		}
		
		inline void DestroyMissile( int index )
		{
			Missiles[index & (MAX_MISSILES-1)].SetIsDone(true);
		}
		
		inline void IncreaseGauge( int value )
		{
			LaserGauge += value;
			if( LaserGauge > LASER_GAUGE_FULL ) LaserGauge = LASER_GAUGE_FULL;
		}
		
		
		s32		x;
		s32		y;
		int		Score;
		int 	Lives;
		int		State;		// Auto or manual
		int 	Mode;		// Enter Exit
		bool	IsAlive;
		int		NewLifeCounter;	// when to spawn another star
		int		StateCounter;	// invincible
		
	private:
		s32		Dx;
		s32		Dy;
		int		BurstCounter;
		int 	StartFrame;
		int 	FlipMode;
		int		Frame;
		int 	Counter;
		int 	Direction;
		bool	Active;
		int 	MissileSpawnDelay;
		int 	MissileFrameOffset;
		int 	MissileRotation;
		int 	NumMissiles;
		int		LaserReady;
		int 	LaserGauge;
		
		HomingMissile Missiles[MAX_MISSILES];
		
		
		void Bounce();
		void Limit();
		
};





#endif
